from matplotlib.text import Text
from typing import Dict, Union, Callable

from mpl_format.compound_types import FontSize
from mpl_format.text.text_utils import wrap_text, remove_parenthesized_text, HORIZONTAL_ALIGNMENTS, VERTICAL_ALIGNMENTS


class TextFormatter(object):

    def __init__(self, text: Text):
        """
        Create a new TextFormatter for a matplotlib Text instance.

        :param text: The matplotlib Text instance to wrap.
        """
        self._text: Text = text

    @property
    def text(self) -> Text:
        """
        Return the wrapped matplotlib Text instance.
        """
        return self._text

    def to_string(self) -> str:
        """
        Return the text of the wrapped matplotlib Text instance.
        """
        return self._text.get_text()

    def wrap(self, max_width: int) -> 'TextFormatter':
        """
        Wrap the text with new lines if it exceeds a given width of characters.

        :param max_width: The maximum character width per line.
        """
        wrapped = wrap_text(self.to_string(), max_width=max_width)
        self._text.set_text(wrapped)
        return self

    def rotate(
            self, rotation: Union[str, int], how: str = 'absolute'
    ) -> 'TextFormatter':
        """
        Set the rotation of the text.

        :param rotation: The rotation value to set in degrees, 'horizontal'
                         or 'vertical'.
        :param how: 'absolute' or 'relative'
        """
        if how == 'relative' and not isinstance(rotation, str):
            self._text.set_rotation(self._text.get_rotation() + rotation)
        elif how == 'absolute':
            self._text.set_rotation(rotation)
        else:
            raise ValueError("`how` must be 'absolute' or 'relative'")
        return self

    def map(
            self, mapping: Union[Dict[str, str], Callable[[str], str]]
    ) -> 'TextFormatter':
        """
        Replace text using a dictionary or function.

        :param mapping: Mappings to replace text.
        """
        if isinstance(mapping, dict):
            if self.to_string() in mapping.keys():
                self._text.set_text(mapping[self.to_string()])
        elif callable(mapping):
            self._text.set_text(mapping(self.to_string()))
        else:
            raise TypeError('mapping must be a dict or callable')
        return self

    def remove_parenthesized_text(self) -> 'TextFormatter':
        """
        Remove any text inside parentheses, along with the parentheses.
        """
        self._text.set_text(remove_parenthesized_text(self.to_string()))
        return self

    def replace(self, old: str, new: str) -> 'TextFormatter':
        """
        Replace old text with new text.

        :param old: The text to remove.
        :param new: The text to insert.
        """
        self._text.set_text(
            self.to_string().replace(old, new)
        )
        return self

    def set_text(self, text: str) -> 'TextFormatter':
        """
        Set the text string.
        """
        self._text.set_text(text)
        return self

    def clear(self) -> 'TextFormatter':
        """
        Clear the text string.
        """
        self.set_text('')
        return self

    # region set size

    def set_size(self, font_size: FontSize) -> 'TextFormatter':
        """
        Set the font size for the Text.
        """
        self._text.set_fontsize(font_size)
        return self
    
    def set_size_xx_small(self) -> 'TextFormatter':
        self.set_size('xx-small')
        return self

    def set_size_x_small(self) -> 'TextFormatter':
        self.set_size('x-small')
        return self

    def set_size_small(self) -> 'TextFormatter':
        self.set_size('small')
        return self

    def set_size_medium(self) -> 'TextFormatter':
        self.set_size('medium')
        return self

    def set_size_large(self) -> 'TextFormatter':
        self.set_size('large')
        return self

    def set_size_x_large(self) -> 'TextFormatter':
        self.set_size('x-large')
        return self

    def set_size_xx_large(self) -> 'TextFormatter':
        self.set_size('xx-large')
        return self

    def set_size_larger(self) -> 'TextFormatter':
        self.set_size('larger')
        return self

    def set_size_smaller(self) -> 'TextFormatter':
        self.set_size('smaller')
        return self

    # endregion

    # region set font family

    def set_font_family(self, font_name: str) -> 'TextFormatter':
        """
        Set the font family to the given font name.

        :param font_name: Name of the font to set.
        """
        self._text.set_fontfamily(fontname=font_name)
        return self

    def set_font_family_serif(self) -> 'TextFormatter':
        """
        Set the font family to 'serif'
        """
        self.set_font_family('serif')
        return self

    def set_font_family_sans_serif(self) -> 'TextFormatter':
        """
        Set the font family to 'sans-serif'
        """
        self.set_font_family('sans-serif')
        return self

    def set_font_family_cursive(self) -> 'TextFormatter':
        """
        Set the font family to 'cursive'
        """
        self.set_font_family('cursive')
        return self

    def set_font_family_fantasy(self) -> 'TextFormatter':
        """
        Set the font family to 'fantasy'
        """
        self.set_font_family('fantasy')
        return self

    def set_font_family_monospace(self) -> 'TextFormatter':
        """
        Set the font family to 'monospace'
        """
        self.set_font_family('monospace')
        return self

    def set_font_family_benton_sans_f(self) -> 'TextFormatter':
        """
        Set the font family to 'BentonSansF'
        """
        self.set_font_family('BentonSansF')
        return self

    def set_font_family_quarto(self) -> 'TextFormatter':
        """
        Set the font family to 'Quarto'
        """
        self.set_font_family('Quarto')
        return self

    def set_font_family_calibri(self) -> 'TextFormatter':
        """
        Set the font family to 'Calibri'
        """
        self.set_font_family('Calibri')
        return self

    # endregion

    # region set alignment

    def set_ha(self, alignment: str) -> 'TextFormatter':

        if alignment not in HORIZONTAL_ALIGNMENTS:
            raise ValueError(
                f'alignment must be one of {HORIZONTAL_ALIGNMENTS}'
            )
        self._text.set_horizontalalignment(align=alignment)
        return self

    def set_ha_left(self) -> 'TextFormatter':

        return self.set_ha('left')

    def set_ha_center(self) -> 'TextFormatter':

        return self.set_ha('center')

    def set_ha_right(self) -> 'TextFormatter':

        return self.set_ha('right')

    def set_va(self, alignment: str) -> 'TextFormatter':

        if alignment not in VERTICAL_ALIGNMENTS:
            raise ValueError(
                f'alignment must be one of {VERTICAL_ALIGNMENTS}'
            )
        self._text.set_verticalalignment(align=alignment)
        return self

    def set_va_top(self) -> 'TextFormatter':

        return self.set_va('top')

    def set_va_center(self) -> 'TextFormatter':

        return self.set_va('center')

    def set_va_bottom(self) -> 'TextFormatter':

        return self.set_va('bottom')

    def set_va_baseline(self) -> 'TextFormatter':

        return self.set_va('baseline')

    def set_va_center_baseline(self) -> 'TextFormatter':

        return self.set_va('center_baseline')

    # endregion


import matplotlib.pyplot as plt
from matplotlib.axes import Axes


def new_axes(width: int = None, height: int = None,
             constrained_layout: bool = False) -> Axes:
    """
    Return new matplotlib axes.
    """
    width = width or 16
    height = height or 9
    _, ax = plt.subplots(
        figsize=(width, height),
        constrained_layout=constrained_layout
    )
    return ax



from typing import Iterable, Union, List

from matplotlib.colors import to_rgb

from compound_types.built_ins import FloatOrFloatIterable
from mpl_format.compound_types import Color


def cross_fade(
        from_color: Color, to_color: Color, amount: FloatOrFloatIterable,
) -> Union[Color, List[Color]]:
    """
    Return a new color which fades amount proportion of the way between the 2
    colors.

    :param from_color: The color to fade from.
    :param to_color: The color to fade to.
    :param amount: The amount to fade by, from 0.0 to 1.0
    """
    if isinstance(amount, Iterable):
        return [
            cross_fade(from_color, to_color, amt)
            for amt in amount
        ]
    if isinstance(from_color, str):
        from_color = to_rgb(from_color)
    if isinstance(to_color, str):
        to_color = to_rgb(to_color)

    return tuple([from_value + amount * (to_value - from_value)
                  for from_value, to_value in zip(from_color, to_color)])


def blacken(
        color: Color, amount: FloatOrFloatIterable
) -> Union[Color, List[Color]]:

    return cross_fade(from_color=color, to_color='black', amount=amount)


def whiten(
        color: Color, amount: FloatOrFloatIterable
) -> Union[Color, List[Color]]:

    return cross_fade(from_color=color, to_color='white', amount=amount)



from distutils.core import setup
from setuptools import find_packages

setup(
    name='mpl_format',
    packages=find_packages(),
    version='0.285',
    license='MIT',
    description='Library for easier formatting of matplotlib plots written in '
                'a functional style.',
    author='Vahndi Minah',
    url='https://github.com/vahndi/mpl-format',
    keywords=['matplotlib'],
    install_requires=[
        'compound-types',
        'matplotlib',
        'numpy',
        'pandas',
        'seaborn',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3'
    ],
)



from enum import Enum


class LINE_STYLE(Enum):

    solid = 1
    dashed = 2
    dashdot = 3
    dotted = 4


class CAP_STYLE(Enum):

    butt = 1
    round = 2
    projecting = 3


class JOIN_STYLE(Enum):

    miter = 1
    round = 2
    bevel = 3


class ARROW_STYLE(Enum):

    Curve = 1
    CurveB = 2
    BracketB = 3
    CurveFilledB = 4
    CurveA = 5
    CurveAB = 6
    CurveFilledA = 7
    CurveFilledAB = 8
    BracketA = 9
    BracketAB = 10
    Fancy = 11
    Simple = 12
    Wedge = 13
    BarAB = 14

    def get_name(self):

        return {
            'Curve': '-',
            'CurveB': '->',
            'BracketB': '-[',
            'CurveFilledB': '-|>',
            'CurveA': '<-',
            'CurveAB': '<->',
            'CurveFilledA': '<|-',
            'CurveFilledAB': '<|-|>',
            'BracketA': ']-',
            'BracketAB': ']-[',
            'Fancy': 'fancy',
            'Simple': 'simple',
            'Wedge': 'wedge',
            'BarAB': '|-|',
        }[self.name]


class CONNECTION_STYLE(Enum):

    angle = 1
    angle3 = 2
    arc = 3
    arc3 = 4
    bar = 5


class DRAW_STYLE(Enum):

    default = 0
    steps = 1
    steps_pre = 2
    steps_mid = 3
    steps_post = 4

    def get_name(self):

        return {
            'default': 'default',
            'steps': 'steps',
            'steps_pre': 'steps-pre',
            'steps_mid': 'steps-mid',
            'steps_post': 'steps-post'
        }[self.name]


class MARKER_STYLE(Enum):

    nothing = 0
    point = 1
    pixel = 2
    circle = 3
    triangle_down = 4
    triangle_up = 5
    triangle_left = 6
    triangle_right = 7
    octagon = 8
    square = 9
    pentagon = 10
    star = 11
    hexagon_1 = 12
    hexagon_2 = 13
    plus = 14
    cross = 15
    diamond = 16
    diamond_thin = 17
    v_line = 18
    h_line = 19
    plus_filled = 20
    cross_filled = 21
    tick_left = 22
    tick_right = 23
    tick_up = 24
    tick_down = 25
    caret_left = 26
    caret_right = 27
    caret_up = 28
    caret_down = 29
    caret_left_base = 30
    caret_right_base = 31
    caret_up_base = 32
    caret_down_base = 33

    def get_name(self):

        return {
            'nothing': 'nothing',
            'point': 'point',
            'pixel': 'pixel',
            'circle': 'circle',
            'triangle_down': 'triangle_down',
            'triangle_up': 'triangle_up',
            'triangle_left': 'triangle_left',
            'triangle_right': 'triangle_right',
            'octagon': 'octagon',
            'square': 'square',
            'pentagon': 'pentagon',
            'star': 'star',
            'hexagon_1': 'hexagon1',
            'hexagon_2': 'hexagon2',
            'plus': 'plus',
            'cross': 'x',
            'diamond': 'diamond',
            'diamond_thin': 'thin_diamond',
            'v_line': 'vline',
            'h_line': 'hline',
            'plus_filled': 'plus_filled',
            'cross_filled': 'x_filled',
            'tick_left': 'tickleft',
            'tick_right': 'tickright',
            'tick_up': 'tickup',
            'tick_down': 'tickdown',
            'caret_left': 'caretleft',
            'caret_right': 'caretright',
            'caret_up': 'caretup',
            'caret_down': 'caretdown',
            'caret_left_base': 'caretleftbase',
            'caret_right_base': 'caretrightbase',
            'caret_up_base': 'caretupbase',
            'caret_down_base': 'caretdownbase'
        }[self.name]



from matplotlib.text import Text
from unittest.case import TestCase

from mpl_format.text.text_utils import wrap_text, map_text


class TestTextUtils(TestCase):

    def setUp(self) -> None:

        self.a_to_z = 'abcdefghijklmnopqrstuvwxyz'
        self.digits = '0123456789'
        self.a_to_z__wrapped_10 = 'abcdefghij\nklmnopqrst\nuvwxyz'
        self.a_to_z__wrapped_5 = 'abcde\nfghij\nklmno\npqrst\nuvwxy\nz'
        self.digits__wrapped_5 = '01234\n56789'

    def test_wrap_text__str(self):

        wrapped = wrap_text(text=self.a_to_z, max_width=10)
        self.assertEqual(self.a_to_z__wrapped_10, wrapped)

    def test_wrap_text__text(self):

        text__a_to_z = Text(text=self.a_to_z)
        wrapped = wrap_text(text=text__a_to_z, max_width=10)
        self.assertEqual(self.a_to_z__wrapped_10, wrapped)

    def test_wrap_text__list_str(self):

        expected = [self.a_to_z__wrapped_5, self.digits__wrapped_5]
        wrapped = wrap_text(text=[self.a_to_z, self.digits], max_width=5)
        self.assertEqual(expected, wrapped)

    def test_map_text(self):

        mapping = {'a': 'A', 'b': 'B'}
        self.assertEqual('A', map_text(text='a', mapping=mapping))
        self.assertEqual('B', map_text(text='b', mapping=mapping))
        self.assertEqual('c', map_text(text='c', mapping=mapping))
